package com.pulumi.jumbo;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.AfterEach;

import com.pulumi.core.internal.Internal;
import com.pulumi.jumbo.inputs.ACustomTypeArgs;
import com.pulumi.jumbo.ACustomResource;
import com.pulumi.jumbo.ACustomResourceArgs;
import com.pulumi.Log;
import com.pulumi.test.internal.PulumiTestInternal;
import com.pulumi.test.Mocks;
import com.pulumi.test.TestOptions;

import java.util.Optional;
import java.util.concurrent.CompletableFuture;

import static com.pulumi.test.PulumiTest.extractValue;
import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;

class JumboTest {

    @AfterEach
    void cleanup() {
        PulumiTestInternal.cleanup();
    }

    @Test
    void testJumboCustomType() {
        var test = PulumiTestInternal.builder()
                .options(TestOptions.builder()
                        .preview(true)
                        .build())
                .mocks(new JumboMocks())
                .build();

        var result = test.runTest(ctx -> {
            var o = ACustomTypeArgs.builder()
                    .bar1("bar1")
                    .bar10("bar10")
                    .bar100("bar100")
                    .bar101("bar101")
                    .bar102("bar102")
                    .bar103("bar103")
                    .bar104("bar104")
                    .bar105("bar105")
                    .bar106("bar106")
                    .bar107("bar107")
                    .bar108("bar108")
                    .bar109("bar109")
                    .bar11("bar11")
                    .bar110("bar110")
                    .bar111("bar111")
                    .bar112("bar112")
                    .bar113("bar113")
                    .bar114("bar114")
                    .bar115("bar115")
                    .bar116("bar116")
                    .bar117("bar117")
                    .bar118("bar118")
                    .bar119("bar119")
                    .bar12("bar12")
                    .bar120("bar120")
                    .bar121("bar121")
                    .bar122("bar122")
                    .bar123("bar123")
                    .bar124("bar124")
                    .bar125("bar125")
                    .bar126("bar126")
                    .bar127("bar127")
                    .bar128("bar128")
                    .bar129("bar129")
                    .bar13("bar13")
                    .bar130("bar130")
                    .bar131("bar131")
                    .bar132("bar132")
                    .bar133("bar133")
                    .bar134("bar134")
                    .bar135("bar135")
                    .bar136("bar136")
                    .bar137("bar137")
                    .bar138("bar138")
                    .bar139("bar139")
                    .bar14("bar14")
                    .bar140("bar140")
                    .bar141("bar141")
                    .bar142("bar142")
                    .bar143("bar143")
                    .bar144("bar144")
                    .bar145("bar145")
                    .bar146("bar146")
                    .bar147("bar147")
                    .bar148("bar148")
                    .bar149("bar149")
                    .bar15("bar15")
                    .bar150("bar150")
                    .bar151("bar151")
                    .bar152("bar152")
                    .bar153("bar153")
                    .bar154("bar154")
                    .bar155("bar155")
                    .bar156("bar156")
                    .bar157("bar157")
                    .bar158("bar158")
                    .bar159("bar159")
                    .bar16("bar16")
                    .bar160("bar160")
                    .bar161("bar161")
                    .bar162("bar162")
                    .bar163("bar163")
                    .bar164("bar164")
                    .bar165("bar165")
                    .bar166("bar166")
                    .bar167("bar167")
                    .bar168("bar168")
                    .bar169("bar169")
                    .bar17("bar17")
                    .bar170("bar170")
                    .bar171("bar171")
                    .bar172("bar172")
                    .bar173("bar173")
                    .bar174("bar174")
                    .bar175("bar175")
                    .bar176("bar176")
                    .bar177("bar177")
                    .bar178("bar178")
                    .bar179("bar179")
                    .bar18("bar18")
                    .bar180("bar180")
                    .bar181("bar181")
                    .bar182("bar182")
                    .bar183("bar183")
                    .bar184("bar184")
                    .bar185("bar185")
                    .bar186("bar186")
                    .bar187("bar187")
                    .bar188("bar188")
                    .bar189("bar189")
                    .bar19("bar19")
                    .bar190("bar190")
                    .bar191("bar191")
                    .bar192("bar192")
                    .bar193("bar193")
                    .bar194("bar194")
                    .bar195("bar195")
                    .bar196("bar196")
                    .bar197("bar197")
                    .bar198("bar198")
                    .bar199("bar199")
                    .bar2("bar2")
                    .bar20("bar20")
                    .bar200("bar200")
                    .bar201("bar201")
                    .bar202("bar202")
                    .bar203("bar203")
                    .bar204("bar204")
                    .bar205("bar205")
                    .bar206("bar206")
                    .bar207("bar207")
                    .bar208("bar208")
                    .bar209("bar209")
                    .bar21("bar21")
                    .bar210("bar210")
                    .bar211("bar211")
                    .bar212("bar212")
                    .bar213("bar213")
                    .bar214("bar214")
                    .bar215("bar215")
                    .bar216("bar216")
                    .bar217("bar217")
                    .bar218("bar218")
                    .bar219("bar219")
                    .bar22("bar22")
                    .bar220("bar220")
                    .bar221("bar221")
                    .bar222("bar222")
                    .bar223("bar223")
                    .bar224("bar224")
                    .bar225("bar225")
                    .bar226("bar226")
                    .bar227("bar227")
                    .bar228("bar228")
                    .bar229("bar229")
                    .bar23("bar23")
                    .bar230("bar230")
                    .bar231("bar231")
                    .bar232("bar232")
                    .bar233("bar233")
                    .bar234("bar234")
                    .bar235("bar235")
                    .bar236("bar236")
                    .bar237("bar237")
                    .bar238("bar238")
                    .bar239("bar239")
                    .bar24("bar24")
                    .bar240("bar240")
                    .bar241("bar241")
                    .bar242("bar242")
                    .bar243("bar243")
                    .bar244("bar244")
                    .bar245("bar245")
                    .bar246("bar246")
                    .bar247("bar247")
                    .bar248("bar248")
                    .bar249("bar249")
                    .bar25("bar25")
                    .bar250("bar250")
                    .bar251("bar251")
                    .bar252("bar252")
                    .bar253("bar253")
                    .bar254("bar254")
                    .bar255("bar255")
                    .bar256("bar256")
                    .bar26("bar26")
                    .bar27("bar27")
                    .bar28("bar28")
                    .bar29("bar29")
                    .bar3("bar3")
                    .bar30("bar30")
                    .bar31("bar31")
                    .bar32("bar32")
                    .bar33("bar33")
                    .bar34("bar34")
                    .bar35("bar35")
                    .bar36("bar36")
                    .bar37("bar37")
                    .bar38("bar38")
                    .bar39("bar39")
                    .bar4("bar4")
                    .bar40("bar40")
                    .bar41("bar41")
                    .bar42("bar42")
                    .bar43("bar43")
                    .bar44("bar44")
                    .bar45("bar45")
                    .bar46("bar46")
                    .bar47("bar47")
                    .bar48("bar48")
                    .bar49("bar49")
                    .bar5("bar5")
                    .bar50("bar50")
                    .bar51("bar51")
                    .bar52("bar52")
                    .bar53("bar53")
                    .bar54("bar54")
                    .bar55("bar55")
                    .bar56("bar56")
                    .bar57("bar57")
                    .bar58("bar58")
                    .bar59("bar59")
                    .bar6("bar6")
                    .bar60("bar60")
                    .bar61("bar61")
                    .bar62("bar62")
                    .bar63("bar63")
                    .bar64("bar64")
                    .bar65("bar65")
                    .bar66("bar66")
                    .bar67("bar67")
                    .bar68("bar68")
                    .bar69("bar69")
                    .bar7("bar7")
                    .bar70("bar70")
                    .bar71("bar71")
                    .bar72("bar72")
                    .bar73("bar73")
                    .bar74("bar74")
                    .bar75("bar75")
                    .bar76("bar76")
                    .bar77("bar77")
                    .bar78("bar78")
                    .bar79("bar79")
                    .bar8("bar8")
                    .bar80("bar80")
                    .bar81("bar81")
                    .bar82("bar82")
                    .bar83("bar83")
                    .bar84("bar84")
                    .bar85("bar85")
                    .bar86("bar86")
                    .bar87("bar87")
                    .bar88("bar88")
                    .bar89("bar89")
                    .bar9("bar9")
                    .bar90("bar90")
                    .bar91("bar91")
                    .bar92("bar92")
                    .bar93("bar93")
                    .bar94("bar94")
                    .bar95("bar95")
                    .bar96("bar96")
                    .bar97("bar97")
                    .bar98("bar98")
                    .bar99("bar99")
                    .build();

            new ACustomResource("testResource", ACustomResourceArgs.builder()
                    .foo(o)
                    .bar1("bar1")
                    .bar10("bar10")
                    .bar100("bar100")
                    .bar101("bar101")
                    .bar102("bar102")
                    .bar103("bar103")
                    .bar104("bar104")
                    .bar105("bar105")
                    .bar106("bar106")
                    .bar107("bar107")
                    .bar108("bar108")
                    .bar109("bar109")
                    .bar11("bar11")
                    .bar110("bar110")
                    .bar111("bar111")
                    .bar112("bar112")
                    .bar113("bar113")
                    .bar114("bar114")
                    .bar115("bar115")
                    .bar116("bar116")
                    .bar117("bar117")
                    .bar118("bar118")
                    .bar119("bar119")
                    .bar12("bar12")
                    .bar120("bar120")
                    .bar121("bar121")
                    .bar122("bar122")
                    .bar123("bar123")
                    .bar124("bar124")
                    .bar125("bar125")
                    .bar126("bar126")
                    .bar127("bar127")
                    .bar128("bar128")
                    .bar129("bar129")
                    .bar13("bar13")
                    .bar130("bar130")
                    .bar131("bar131")
                    .bar132("bar132")
                    .bar133("bar133")
                    .bar134("bar134")
                    .bar135("bar135")
                    .bar136("bar136")
                    .bar137("bar137")
                    .bar138("bar138")
                    .bar139("bar139")
                    .bar14("bar14")
                    .bar140("bar140")
                    .bar141("bar141")
                    .bar142("bar142")
                    .bar143("bar143")
                    .bar144("bar144")
                    .bar145("bar145")
                    .bar146("bar146")
                    .bar147("bar147")
                    .bar148("bar148")
                    .bar149("bar149")
                    .bar15("bar15")
                    .bar150("bar150")
                    .bar151("bar151")
                    .bar152("bar152")
                    .bar153("bar153")
                    .bar154("bar154")
                    .bar155("bar155")
                    .bar156("bar156")
                    .bar157("bar157")
                    .bar158("bar158")
                    .bar159("bar159")
                    .bar16("bar16")
                    .bar160("bar160")
                    .bar161("bar161")
                    .bar162("bar162")
                    .bar163("bar163")
                    .bar164("bar164")
                    .bar165("bar165")
                    .bar166("bar166")
                    .bar167("bar167")
                    .bar168("bar168")
                    .bar169("bar169")
                    .bar17("bar17")
                    .bar170("bar170")
                    .bar171("bar171")
                    .bar172("bar172")
                    .bar173("bar173")
                    .bar174("bar174")
                    .bar175("bar175")
                    .bar176("bar176")
                    .bar177("bar177")
                    .bar178("bar178")
                    .bar179("bar179")
                    .bar18("bar18")
                    .bar180("bar180")
                    .bar181("bar181")
                    .bar182("bar182")
                    .bar183("bar183")
                    .bar184("bar184")
                    .bar185("bar185")
                    .bar186("bar186")
                    .bar187("bar187")
                    .bar188("bar188")
                    .bar189("bar189")
                    .bar19("bar19")
                    .bar190("bar190")
                    .bar191("bar191")
                    .bar192("bar192")
                    .bar193("bar193")
                    .bar194("bar194")
                    .bar195("bar195")
                    .bar196("bar196")
                    .bar197("bar197")
                    .bar198("bar198")
                    .bar199("bar199")
                    .bar2("bar2")
                    .bar20("bar20")
                    .bar200("bar200")
                    .bar201("bar201")
                    .bar202("bar202")
                    .bar203("bar203")
                    .bar204("bar204")
                    .bar205("bar205")
                    .bar206("bar206")
                    .bar207("bar207")
                    .bar208("bar208")
                    .bar209("bar209")
                    .bar21("bar21")
                    .bar210("bar210")
                    .bar211("bar211")
                    .bar212("bar212")
                    .bar213("bar213")
                    .bar214("bar214")
                    .bar215("bar215")
                    .bar216("bar216")
                    .bar217("bar217")
                    .bar218("bar218")
                    .bar219("bar219")
                    .bar22("bar22")
                    .bar220("bar220")
                    .bar221("bar221")
                    .bar222("bar222")
                    .bar223("bar223")
                    .bar224("bar224")
                    .bar225("bar225")
                    .bar226("bar226")
                    .bar227("bar227")
                    .bar228("bar228")
                    .bar229("bar229")
                    .bar23("bar23")
                    .bar230("bar230")
                    .bar231("bar231")
                    .bar232("bar232")
                    .bar233("bar233")
                    .bar234("bar234")
                    .bar235("bar235")
                    .bar236("bar236")
                    .bar237("bar237")
                    .bar238("bar238")
                    .bar239("bar239")
                    .bar24("bar24")
                    .bar240("bar240")
                    .bar241("bar241")
                    .bar242("bar242")
                    .bar243("bar243")
                    .bar244("bar244")
                    .bar245("bar245")
                    .bar246("bar246")
                    .bar247("bar247")
                    .bar248("bar248")
                    .bar249("bar249")
                    .bar25("bar25")
                    .bar250("bar250")
                    .bar251("bar251")
                    .bar252("bar252")
                    .bar253("bar253")
                    .bar254("bar254")
                    .bar255("bar255")
                    .bar256("bar256")
                    .bar26("bar26")
                    .bar27("bar27")
                    .bar28("bar28")
                    .bar29("bar29")
                    .bar3("bar3")
                    .bar30("bar30")
                    .bar31("bar31")
                    .bar32("bar32")
                    .bar33("bar33")
                    .bar34("bar34")
                    .bar35("bar35")
                    .bar36("bar36")
                    .bar37("bar37")
                    .bar38("bar38")
                    .bar39("bar39")
                    .bar4("bar4")
                    .bar40("bar40")
                    .bar41("bar41")
                    .bar42("bar42")
                    .bar43("bar43")
                    .bar44("bar44")
                    .bar45("bar45")
                    .bar46("bar46")
                    .bar47("bar47")
                    .bar48("bar48")
                    .bar49("bar49")
                    .bar5("bar5")
                    .bar50("bar50")
                    .bar51("bar51")
                    .bar52("bar52")
                    .bar53("bar53")
                    .bar54("bar54")
                    .bar55("bar55")
                    .bar56("bar56")
                    .bar57("bar57")
                    .bar58("bar58")
                    .bar59("bar59")
                    .bar6("bar6")
                    .bar60("bar60")
                    .bar61("bar61")
                    .bar62("bar62")
                    .bar63("bar63")
                    .bar64("bar64")
                    .bar65("bar65")
                    .bar66("bar66")
                    .bar67("bar67")
                    .bar68("bar68")
                    .bar69("bar69")
                    .bar7("bar7")
                    .bar70("bar70")
                    .bar71("bar71")
                    .bar72("bar72")
                    .bar73("bar73")
                    .bar74("bar74")
                    .bar75("bar75")
                    .bar76("bar76")
                    .bar77("bar77")
                    .bar78("bar78")
                    .bar79("bar79")
                    .bar8("bar8")
                    .bar80("bar80")
                    .bar81("bar81")
                    .bar82("bar82")
                    .bar83("bar83")
                    .bar84("bar84")
                    .bar85("bar85")
                    .bar86("bar86")
                    .bar87("bar87")
                    .bar88("bar88")
                    .bar89("bar89")
                    .bar9("bar9")
                    .bar90("bar90")
                    .bar91("bar91")
                    .bar92("bar92")
                    .bar93("bar93")
                    .bar94("bar94")
                    .bar95("bar95")
                    .bar96("bar96")
                    .bar97("bar97")
                    .bar98("bar98")
                    .bar99("bar99")
                    .build()
            );
        }).throwOnError();

        var resource = result.resources().stream()
                .filter(r -> r.getResourceName().equals("testResource"))
                .filter(r -> r instanceof ACustomResource)
                .map(r -> (ACustomResource) r)
                .findFirst()
                .orElse(null);
        assertThat(resource).isNotNull();

        var foo = extractValue(resource.foo());
        assertThat(foo).isPresent();
        assertThat(foo.get().bar1()).hasValue("bar1");
    }

    public static class JumboMocks implements Mocks {
        @Override
        public CompletableFuture<ResourceResult> newResourceAsync(ResourceArgs args) {
            if ("jumbo::ACustomResource".equals(args.type)) {
                return CompletableFuture.completedFuture(
                        ResourceResult.of(Optional.of(args.name + "_id"), args.inputs)
                );
            }
            throw new IllegalArgumentException(String.format("Unknown resource '%s'", args.type));
        }
    }
}
